package AlgLin;

/**
 * @author Hamadou BA  
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp2-ps-resolution-de-systemes-lineaires.git  
 *  
 * Classe representant un systeme lineaire diagonal et permettant sa resolution.  
 */

public class SysDiagonal extends SysLin {

    public SysDiagonal(Matrice matrice, Vecteur vecteur) throws IrregularSysLinException {
        super(matrice, vecteur);
    }

    @Override
    public Vecteur resolution() throws IrregularSysLinException {
        int taille = matriceSystem.nbLigne();
        Vecteur solution = new Vecteur(taille);

        for (int indice = 0; indice < taille; indice++) {
            double coefficientDiagonal = matriceSystem.getCoeff(indice, indice);
            if (Math.abs(coefficientDiagonal) < Matrice.EPSILON) {
                throw new IrregularSysLinException("Systeme diagonal invalide.");
            }
            double valeur = secondMembre.getCoeff(indice) / coefficientDiagonal;
            solution.remplacecoef(indice, valeur);
        }
        return solution;
    }

    public static void main(String[] args) throws IrregularSysLinException {
        System.out.println("Test de la resolution d'un systeme diagonal");

        double[] valeursVecteur = {8, 10, 12};
        double[][] valeursMatrice = {
            {4, 0, 0},
            {0, 5, 0},
            {0, 0, 6}
        };

        Vecteur vecteurB = new Vecteur(valeursVecteur);
        Matrice matriceA = new Matrice(valeursMatrice);
        SysLin systemeDiagonal = new SysDiagonal(matriceA, vecteurB);
        Vecteur solutionX = systemeDiagonal.resolution();

        System.out.println("Solution obtenue : " + solutionX);

        Matrice vecteurBNegatif = new Matrice(vecteurB.getTaille(), 1);
        for (int i = 0; i < vecteurB.getTaille(); i++) {
            vecteurBNegatif.remplacecoef(i, 0, -vecteurB.getCoeff(i));
        }

        Matrice resultatAx = Matrice.produit(matriceA, solutionX);
        System.out.println("Resultat de A * x : \n" + resultatAx);

        Matrice difference = Matrice.addition(resultatAx, vecteurBNegatif);
        System.out.println("Resultat de A * x - b : \n" + difference);

        Vecteur vecteurNorme = new Vecteur(difference.nbLigne());
        for (int i = 0; i < difference.nbLigne(); i++) {
            vecteurNorme.remplacecoef(i, difference.getCoeff(i, 0));
        }

        double nL1 = Vecteur.normeL1(vecteurNorme);
        System.out.println("Norme L1 : " + nL1);
        if (nL1 <= 0.0 || nL1 > Matrice.EPSILON) {
            System.out.println("=> La norme L1 est nulle ou tres petite");
        }

        double nL2 = Vecteur.normeL2(vecteurNorme);
        System.out.println("Norme L2 : " + nL2);
        if (nL2 <= 0.0 || nL2 > Matrice.EPSILON) {
            System.out.println("=> La norme L2 est nulle ou tres petite");
        }

        double nLinf = Vecteur.normeLinfini(vecteurNorme);
        System.out.println("Norme Linfini : " + nLinf);
        if (nLinf <= 0.0 || nLinf > Matrice.EPSILON) {
            System.out.println("=> La norme Linfini est nulle ou tres petite");
        }
    }
}
