package AlgLin;

/**
 * @author Hamadou BA  
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp2-ps-resolution-de-systemes-lineaires.git  
 *  
 * Classe représentant un système linéaire triangulaire inférieur avec une diagonale unitaire.  
 */
public class SysTriangInfUnite extends SystTriangInf {

    public SysTriangInfUnite(Matrice m, Vecteur v) throws IrregularSysLinException {
        super(m, v);
        int taille = m.nbLigne();
        for (int i = 0; i < taille; i++) {
            if (Math.abs(m.getCoeff(i, i) - 1.0) > Matrice.EPSILON) {
                throw new IrregularSysLinException("Tous les elements diagonaux doivent etre egaux a 1.");
            }
        }
    }

    public static void main(String... args) throws IrregularSysLinException {
        double[][] matriceTest = {
            {1, 0, 0, 0},
            {2, 1, 0, 0},
            {3, 4, 1, 0},
            {5, 6, 7, 1}
        };
        Matrice matriceA = new Matrice(matriceTest);
        Vecteur vecteurB = new Vecteur(new double[]{4, 5, 6, 7});

        SysTriangInfUnite systeme = new SysTriangInfUnite(matriceA, vecteurB);
        Vecteur solutionX = systeme.resolution();

        System.out.println("Matrice A : \n" + matriceA);
        System.out.println("Solution x : " + solutionX);

        Matrice vecteurBNegatif = new Matrice(vecteurB.getTaille(), 1);
        for (int i = 0; i < vecteurB.getTaille(); i++) {
            vecteurBNegatif.remplacecoef(i, 0, -vecteurB.getCoeff(i));
        }

        Matrice Ax = Matrice.produit(matriceA, solutionX);
        System.out.println("Produit Ax : \n" + Ax);

        Matrice difference = Matrice.addition(Ax, vecteurBNegatif);
        System.out.println("Résultat Ax - b : " + difference);

        Vecteur normeVecteur = new Vecteur(difference.nbLigne());
        for (int i = 0; i < difference.nbLigne(); i++) {
            normeVecteur.remplacecoef(i, difference.getCoeff(i, 0));
        }

        double nL1 = Vecteur.normeL1(normeVecteur);
        System.out.println("Norme L1 : " + nL1);
        if (nL1 > Matrice.EPSILON) {
            System.out.println("=> La norme L1 n'est pas negligeable.");
        }

        double nL2 = Vecteur.normeL2(normeVecteur);
        System.out.println("Norme L2 : " + nL2);
        if (nL2 > Matrice.EPSILON) {
            System.out.println("=> La norme L2 n'est pas negligeable.");
        }

        double nLinf = Vecteur.normeLinfini(normeVecteur);
        System.out.println("Norme Linfini : " + nLinf);
        if (nLinf > Matrice.EPSILON) {
            System.out.println("=> La norme Linfini n'est pas negligeable.");
        }
    }
}
