package AlgLin;

/**
 * @author Hamadou BA  
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp2-ps-resolution-de-systemes-lineaires.git  
 *  
 * Classe representant un systeme lineaire sous forme triangulaire superieure et sa resolution.
 */
public class SysTriangSup extends SysLin {

    public SysTriangSup(Matrice matrice, Vecteur vecteur) throws IrregularSysLinException {
        super(matrice, vecteur);
    }

    @Override
    public Vecteur resolution() throws IrregularSysLinException {
        Matrice matriceSysteme = getMatriceSystem();
        Vecteur solution = new Vecteur(matriceSysteme.nbLigne());

        for (int ligne = matriceSysteme.nbLigne() - 1; ligne >= 0; ligne--) {
            double somme = 0.0;

            for (int col = ligne + 1; col < matriceSysteme.nbLigne(); col++) {
                somme += matriceSysteme.getCoeff(ligne, col) * solution.getCoeff(col);
            }

            if (Math.abs(matriceSysteme.getCoeff(ligne, ligne)) < Matrice.EPSILON) {
                throw new IrregularSysLinException("Le systeme est singulier : diagonale nulle a la ligne " + ligne);
            }

            solution.remplacecoef(ligne, (secondMembre.getCoeff(ligne) - somme) / matriceSysteme.getCoeff(ligne, ligne));
        }

        return solution;
    }

    public static void main(String[] args) throws IrregularSysLinException {
    	
        System.out.println("Test de la resolution d'un systeme triangulaire superieur");

        double matriceDonnees[][] = { { 3, 1, 2 }, { 0, 7, 4 }, { 0, 0, 8 } };
        Matrice matricePrincipale = new Matrice(matriceDonnees);

        Vecteur vecteurEntree = new Vecteur(new double[] {-3, -1.0/4, 5 });

        SysTriangSup systeme = new SysTriangSup(matricePrincipale, vecteurEntree);

        Vecteur resultat = systeme.resolution();
        System.out.println("Solution x = " + resultat);

        Matrice vecteurNegatif = new Matrice(vecteurEntree.getTaille(), 1);
        for (int i = 0; i < vecteurEntree.getTaille(); i++) {
            vecteurNegatif.remplacecoef(i, 0, vecteurEntree.getCoeff(i));
            vecteurNegatif.remplacecoef(i, 0, -vecteurNegatif.getCoeff(i, 0));
        }

        Matrice produitAx = Matrice.produit(matricePrincipale, resultat);
        System.out.println("Le produit Ax : \n" + produitAx);

        Matrice differenceAx_b = Matrice.addition(produitAx, vecteurNegatif);
        System.out.println("Le résultat Ax - b : " + differenceAx_b);

        Vecteur normeVecteur = new Vecteur(differenceAx_b.nbLigne());
        for (int i = 0; i < differenceAx_b.nbLigne(); i++) {
            normeVecteur.remplacecoef(i, differenceAx_b.getCoeff(i, 0));
        }

        double nL1 = Vecteur.normeL1(normeVecteur);
        System.out.println("Norme L1 : " + nL1);
        if (nL1 <= 0.0 || nL1 > Matrice.EPSILON) {
            System.out.println("=> La norme L1 est nulle ou très petite");
        }

        double nL2 = Vecteur.normeL2(normeVecteur);
        System.out.println("Norme L2 : " + nL2);
        if (nL2 <= 0.0 || nL2 > Matrice.EPSILON) {
            System.out.println("=> La norme L2 est nulle ou très petite");
        }

        double nLinfini = Vecteur.normeLinfini(normeVecteur);
        System.out.println("Norme Linfini : " + nLinfini);
        if (nLinfini <= 0.0 || nLinfini > Matrice.EPSILON) {
            System.out.println("=> La norme Linfini est nulle ou très petite");
        }
    }

}
