package AlgLin;

/**
 * @author Hamadou BA  
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp2-ps-resolution-de-systemes-lineaires.git  
 *  
 * Classe representant un systeme lineaire avec une matrice triangulaire superieure ayant des elements diagonaux egaux a 1.
 */

public class SysTriangSupUnite extends SysTriangSup {

    public SysTriangSupUnite(Matrice matrice, Vecteur vecteur) throws IrregularSysLinException {
        super(matrice, vecteur);
        for (int i = vecteur.getTaille() - 1; i >= 0; i--) {
            if (Math.abs(matrice.getCoeff(i, i) - 1.0) > Matrice.EPSILON) {
                throw new IrregularSysLinException("La diagonale n'est pas egale a 1");
            }
        }
    }

    public static void main(String[] args) throws IrregularSysLinException {
    	
        System.out.println("Test de la resolution d'un systeme triangulaire superieur avec des elements diagonaux egaux a 1");

        double matriceTest[][] = { { 1, 4, 2 }, { 0, 1, 3 }, { 0, 0, 1 } };
        Matrice matriceTestA = new Matrice(matriceTest);
        Vecteur vecteurTestB = new Vecteur(new double[] { 5, 3, 2 });

        SysTriangSupUnite systemeTest = new SysTriangSupUnite(matriceTestA, vecteurTestB);
        Vecteur solutionTestX = systemeTest.resolution();
        System.out.println("Solution x = " + solutionTestX);

        Matrice vecteurInverse = new Matrice(vecteurTestB.getTaille(), 1);
        for (int i = 0; i < vecteurTestB.getTaille(); i++) {
            vecteurInverse.remplacecoef(i, 0, vecteurTestB.getCoeff(i));
            vecteurInverse.remplacecoef(i, 0, -vecteurInverse.getCoeff(i, 0));
        }

        Matrice produitAxTest = Matrice.produit(matriceTestA, solutionTestX);
        System.out.println("Le produit Ax : \n" + produitAxTest);

        Matrice differenceAx_bTest = Matrice.addition(produitAxTest, vecteurInverse);
        System.out.println("Le résultat Ax - b : " + differenceAx_bTest);
        Vecteur normeTest = new Vecteur(differenceAx_bTest.nbLigne());
        for (int i = 0; i < differenceAx_bTest.nbLigne(); i++) {
            normeTest.remplacecoef(i, differenceAx_bTest.getCoeff(i, 0));
        }

        double nL1Test = Vecteur.normeL1(normeTest);
        System.out.println("Norme L1 : " + nL1Test);
        if (nL1Test <= 0.0 || nL1Test > Matrice.EPSILON) {
            System.out.println("=> La norme L1 est nulle ou très petite");
        }

        double nL2Test = Vecteur.normeL2(normeTest);
        System.out.println("Norme L2 : " + nL2Test);
        if (nL2Test <= 0.0 || nL2Test > Matrice.EPSILON) {
            System.out.println("=> La norme L2 est nulle ou très petite");
        }

        double nLinfiniTest = Vecteur.normeLinfini(normeTest);
        System.out.println("Norme Linfini : " + nLinfiniTest);
        if (nLinfiniTest <= 0.0 || nLinfiniTest > Matrice.EPSILON) {
            System.out.println("=> La norme Linfini est nulle ou très petite");
        }
    }
}
