package AlgLin;

/**
 * @author Hamadou BA  
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp2-ps-resolution-de-systemes-lineaires.git  
 *  
 * Classe representant un système lineaire avec une matrice triangulaire inferieure.
 */
public class SystTriangInf extends SysLin {

    public SystTriangInf(Matrice matrice, Vecteur vecteur) throws IrregularSysLinException {
        super(matrice, vecteur);
    }

    @Override
    public Vecteur resolution() throws IrregularSysLinException {
        Matrice matriceSystem = getMatriceSystem();
        Vecteur solution = new Vecteur(matriceSystem.nbLigne());

        for (int i = 0; i < matriceSystem.nbLigne(); i++) {
            double somme = 0.0;
            for (int j = 0; j < i; j++) {
                somme += matriceSystem.getCoeff(i, j) * solution.getCoeff(j);
            }
            if (Math.abs(matriceSystem.getCoeff(i, i)) < Matrice.EPSILON)
                throw new IrregularSysLinException("Probleme avec l'element diagonal");

            solution.remplacecoef(i, (secondMembre.getCoeff(i) - somme) / matriceSystem.getCoeff(i, i));
        }
        return solution;
    }

    public static void main(String... args) throws IrregularSysLinException {
    	
        System.out.println("Test de la resolution d'un systeme lineaire avec une matrice triangulaire inferieure");

        double matriceData[][] = { { 3, 0, 0 }, { 2, 3, 0 }, { 4, 5, 3 } };
        Matrice matriceTest = new Matrice(matriceData);
        Vecteur vecteurTest = new Vecteur(new double[] { 6, 5, 7 });
        SysLin systemeTest = new SystTriangInf(matriceTest, vecteurTest);
        Vecteur solutionTest = systemeTest.resolution();
        System.out.println("Solution du système : x = " + solutionTest);

        Matrice vecteurInverse = new Matrice(vecteurTest.getTaille(), 1);
        for (int i = 0; i < vecteurTest.getTaille(); i++) {
            vecteurInverse.remplacecoef(i, 0, vecteurTest.getCoeff(i));
            vecteurInverse.remplacecoef(i, 0, -vecteurInverse.getCoeff(i, 0));
        }

        Matrice resultatAx = Matrice.produit(matriceTest, solutionTest);
        System.out.println("Le produit Ax : \n" + resultatAx);

        Matrice differenceAx_b = Matrice.addition(resultatAx, vecteurInverse);
        System.out.println("Le resultat Ax - b : " + differenceAx_b);
        Vecteur normeVecteur = new Vecteur(differenceAx_b.nbLigne());
        for (int i = 0; i < differenceAx_b.nbLigne(); i++) {
            normeVecteur.remplacecoef(i, differenceAx_b.getCoeff(i, 0));
        }

        double nL1Test = Vecteur.normeL1(normeVecteur);
        System.out.println("Norme L1 : " + nL1Test);
        if (nL1Test <= 0.0 || nL1Test > Matrice.EPSILON) {
            System.out.println("=> La norme L1 est nulle ou trop petite.");
        }

        double noL2Test = Vecteur.normeL2(normeVecteur);
        System.out.println("Norme L2 : " + noL2Test);
        if (noL2Test <= 0.0 || noL2Test > Matrice.EPSILON) {
            System.out.println("=> La norme L2 est nulle ou trop petite.");
        }

        double nLinfiniTest = Vecteur.normeLinfini(normeVecteur);
        System.out.println("Norme Linfini : " + nLinfiniTest);
        if (nLinfiniTest <= 0.0 || nLinfiniTest > Matrice.EPSILON) {
            System.out.println("=> La norme Linfini est nulle ou trop petite.");
        }
    }
}
