package AlgLin;
/**
 * @author Hamadou BA
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp2-ps-resolution-de-systemes-lineaires.git  
 * 
 * La classe Vecteur represente un vecteur
 * permettant de manipuler et effectuer des calculs sur des vecteurs
 * Cette classe herite de la classe Matrice et fournit des fonctionnalites comme l'affichage, 
 * la lecture a partir de fichiers, la modification de coefficients, et le calcul du produit scalaire
 */

import java.io.File;
import java.io.FileNotFoundException;
import java.util.InputMismatchException;
import java.util.Scanner;

public class Vecteur extends Matrice {

	public Vecteur(int nbligne) {
		super(nbligne, 1);
	}

	
	public Vecteur(double[] tableau) {
		super(tableau.length, 1);
		for (int i = 0; i < tableau.length; i++) {
			this.coefficient[i][0] = tableau[i];
		}

	}

	
	public Vecteur(String fichier)  {
        super(1, 1); 
        try {
            Scanner sc = new Scanner(new File(fichier));
            int taille = sc.nextInt();
            this.coefficient = new double[taille][1];
            for (int i = 0; i < taille; i++) {
                this.coefficient[i][0] = sc.nextDouble();
            }
            sc.close();
		} catch (FileNotFoundException | InputMismatchException e) {
			e.printStackTrace();
		}
    }

	
	public int getTaille() {
		return this.nbLigne();
	}


	public double getCoeff(int pos) {
		return this.getCoeff(pos, 0);
	}

	
	public void remplacecoef(int pos, double valeur) {
		this.remplacecoef(pos, 0, valeur);
	}

	
	public static Vecteur produit(Matrice m, Vecteur v) {
		Vecteur produit = new Vecteur(m.nbLigne());
		for (int i = 0; i < m.nbLigne(); i++) {
			double somme = 0;
			for (int j = 0; j < m.nbColonne(); j++) {
				somme += m.getCoeff(i, j) * v.getCoeff(j);
			}
			produit.remplacecoef(i, somme);
		}
		return produit;
	}

	
	
	public static double produit(Vecteur a, Vecteur b) throws Exception {
		double produit = 0.0;
		for (int i = 0; i < a.getTaille(); i++) {
			produit += a.getCoeff(i) * b.getCoeff(i);
		}
		return produit;
	}

	
	public static double verif_produit(Vecteur a, Vecteur b) throws Exception {
		if (a.getTaille() != b.getTaille())
			throw new Exception("Les tailles des vecteurs ne sont pas égales");
		double produit = 0.0;
		for (int i = 0; i < a.getTaille(); i++) {
			produit += a.getCoeff(i) + b.getCoeff(i);
		}
		return produit;
	}
	
	
	public static Vecteur addition(Vecteur a, Vecteur b) {
		Vecteur somme = new Vecteur(a.getTaille());
		for (int i = 0; i < a.getTaille(); i++) {
			somme.remplacecoef(i, a.getCoeff(i) + b.getCoeff(i));
		}
		return somme;
	}
	
	
	public static Vecteur verif_addition(Vecteur a, Vecteur b) throws Exception {
		if (a.getTaille() != b.getTaille()) {
			throw new Exception("Les tailles des vecteurs ne sont pas égales");
		}
		Vecteur somme = new Vecteur(a.getTaille());
		for (int i = 0; i < a.getTaille(); i++) {
			somme.remplacecoef(i, a.getCoeff(i) + b.getCoeff(i));
		}
		return somme;
	}


	
	public String toString() {
		String vect = "(";
		for (int i = 0; i < this.getTaille(); i++) {
			vect += this.coefficient[i][0];
			if (i != this.getTaille() - 1) {
				vect += ", ";
			}
		}
		vect += ")";
		return vect;
	}


	public static double normeL1(Vecteur vect) {
		double norme = 0.0;
		for (int i = 0; i < vect.getTaille(); i++) {
			norme += Math.abs(vect.getCoeff(i));
		}
		return norme;
	}

	
	public static double normeL2(Vecteur vect) {
		double norme = 0.0;
		for (int i = 0; i < vect.getTaille(); i++) {
			norme += Math.pow(vect.getCoeff(i), 2);
		}
		return Math.sqrt(norme);
	}

	
	public static double normeLinfini(Vecteur vect) {
		double norme = 0.0;
		for (int i = 0; i < vect.getTaille(); i++) {
			norme = Math.max(norme, Math.abs(vect.getCoeff(i)));
		}
		return norme;
	}

	public static void main(String[] args) throws Exception {
		
        double[] tableau1 = {1.0, 2.0, 3.0};
        Vecteur v1 = new Vecteur(tableau1);
        System.out.println("Construction d'un vecteur par affectation d'un tableau : \n");
        System.out.println(v1);
        System.out.println(); 
        
        Vecteur v2 = new Vecteur("./resources/vecteur.txt");
        System.out.println("Construction d'un vecteur par lecture d'un fichier :");
        System.out.println(v2);
        System.out.println();

        Vecteur v3 = new Vecteur(5);
        System.out.println("Construction d'un vecteur de taille 5 :");
        System.out.println(v3);
        System.out.println();

        System.out.println("Taille du vecteur v1 : " + v1.getTaille());
        System.out.println(); 

        System.out.println("Coefficient a la position 1 dans le vecteur v1 : " + v1.getCoeff(1));
        System.out.println(); 


        Vecteur v4 = new Vecteur(new double[]{4.0, 5.0, 6.0});
        double produit = Vecteur.produit(v1, v4);
        System.out.println("Produit scalaire de v1 et v4 :");
        System.out.println(produit);
        System.out.println();

        double produitVerifie = Vecteur.verif_produit(v1, v4);
        System.out.println("Produit scalaire de v1 et v4 (avec verification) :");
        System.out.println(produitVerifie);
	}

}