package tp6.ps;

import org.knowm.xchart.SwingWrapper;
import org.knowm.xchart.XYChart;
import org.knowm.xchart.XYChartBuilder;
import org.knowm.xchart.XYSeries;
import org.knowm.xchart.style.Styler.LegendPosition;
import org.knowm.xchart.style.markers.SeriesMarkers;


public class ChartPlotter {
    
  
    public static void plotPolynomialFit(DataLoader.PointsData pointsData, ModPoly modPoly, String title) {
        XYChart chart = new XYChartBuilder()
            .width(800)
            .height(600)
            .title(title)
            .xAxisTitle("X")
            .yAxisTitle("Y")
            .build();
        
        chart.getStyler().setLegendPosition(LegendPosition.InsideNE);
        chart.getStyler().setDefaultSeriesRenderStyle(XYSeries.XYSeriesRenderStyle.Scatter);
        
        double[] x = pointsData.getX();
        double[] y = pointsData.getY();
        XYSeries pointsSeries = chart.addSeries("Points de support", x, y);
        pointsSeries.setMarker(SeriesMarkers.CIRCLE);
        
        double xMin = Double.MAX_VALUE;
        double xMax = Double.MIN_VALUE;
        for (double xi : x) {
            xMin = Math.min(xMin, xi);
            xMax = Math.max(xMax, xi);
        }
        
        double margin = 0.1 * (xMax - xMin);
        xMin -= margin;
        xMax += margin;
        
        int nbPoints = 100;
        double[] xCurve = new double[nbPoints];
        double[] yCurve = new double[nbPoints];
        
        for (int i = 0; i < nbPoints; i++) {
            xCurve[i] = xMin + i * (xMax - xMin) / (nbPoints - 1);
            yCurve[i] = modPoly.evaluer(xCurve[i]);
        }
        
        XYSeries polynomialSeries = chart.addSeries(
            "Polynôme de degré " + modPoly.getDegre(), 
            xCurve, 
            yCurve
        );
        polynomialSeries.setMarker(SeriesMarkers.NONE);
        polynomialSeries.setXYSeriesRenderStyle(XYSeries.XYSeriesRenderStyle.Line);
        
        new SwingWrapper<>(chart).displayChart();
    }
}