package tp6.ps;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

public class DataLoader {
    
  
    public static class PointsData {
        private double[] x;
        private double[] y;
        
        public PointsData(double[] x, double[] y) {
            this.x = x;
            this.y = y;
        }
        
        public double[] getX() {
            return x;
        }
        
        public double[] getY() {
            return y;
        }
        
        public int getSize() {
            return x.length;
        }
    }
    
  
    public static PointsData loadFromFile(String filePath) throws IOException {
        Path path = Paths.get(filePath);
        List<Double> xList = new ArrayList<>();
        List<Double> yList = new ArrayList<>();
        
        try (BufferedReader reader = Files.newBufferedReader(path)) {
            String line;
            while ((line = reader.readLine()) != null) {
                line = line.trim();
                if (line.isEmpty() || line.startsWith("#")) {
                    continue; 
                }
                
                String[] values = line.split("\\s+");
                if (values.length >= 2) {
                    try {
                        double x = Double.parseDouble(values[0]);
                        double y = Double.parseDouble(values[1]);
                        xList.add(x);
                        yList.add(y);
                    } catch (NumberFormatException e) {
                        System.err.println("Ligne ignorée (format incorrect): " + line);
                    }
                }
            }
        }
        
        double[] x = new double[xList.size()];
        double[] y = new double[yList.size()];
        
        for (int i = 0; i < xList.size(); i++) {
            x[i] = xList.get(i);
            y[i] = yList.get(i);
        }
        
        return new PointsData(x, y);
    }
    

    public static PointsData loadFromResource(String resourcePath) throws IOException {
        List<Double> xList = new ArrayList<>();
        List<Double> yList = new ArrayList<>();
        
        try (InputStream is = DataLoader.class.getClassLoader().getResourceAsStream(resourcePath)) {
            if (is == null) {
                throw new IOException("Resource not found: " + resourcePath);
            }
            
            BufferedReader reader = new BufferedReader(new InputStreamReader(is));
            String line;
            while ((line = reader.readLine()) != null) {
                line = line.trim();
                if (line.isEmpty() || line.startsWith("#")) {
                    continue; // Ignore les lignes vides ou commentées
                }
                
                String[] values = line.split("\\s+");
                if (values.length >= 2) {
                    try {
                        double x = Double.parseDouble(values[0]);
                        double y = Double.parseDouble(values[1]);
                        xList.add(x);
                        yList.add(y);
                    } catch (NumberFormatException e) {
                        System.err.println("Ligne ignorée (format incorrect): " + line);
                    }
                }
            }
        }
        
        // Conversion des listes en tableaux
        double[] x = new double[xList.size()];
        double[] y = new double[yList.size()];
        
        for (int i = 0; i < xList.size(); i++) {
            x[i] = xList.get(i);
            y[i] = yList.get(i);
        }
        
        return new PointsData(x, y);
    }
    
   
    public static void debugResourceContent(String resourcePath) throws IOException {
        try (InputStream is = DataLoader.class.getClassLoader().getResourceAsStream(resourcePath)) {
            if (is == null) {
                System.err.println("Ressource introuvable: " + resourcePath);
                return;
            }
            
            BufferedReader reader = new BufferedReader(new InputStreamReader(is));
            System.out.println("Contenu du fichier " + resourcePath + ":");
            String line;
            int lineNum = 1;
            while ((line = reader.readLine()) != null) {
                System.out.println(lineNum + ": " + line);
                lineNum++;
            }
            System.out.println("Fin du fichier");
        }
    }
}